public class List {

	// soukromá třída - ostatní třídy nevidí do implementace seznamu
	/**
	 * Jeden uzel v seznamu.
	 */
	private static class ListNode {

		// package přístup, třída List přistupuje přímo
		/**
		 * Data uložená v uzlu.
		 */
		Object data;

		/**
		 * Odkaz na následující uzel.
		 */
		ListNode nextNode;

		/**
		 * Vytvoří nový uzel seznamu s danými daty.
		 * 
		 * @param object
		 *            data vkládaná do nového uzlu.
		 */
		public ListNode(Object object) {
			this(object, null);
		}

		/**
		 * Vytvoří nový uzel seznamu s danými daty.
		 * 
		 * @param object
		 *            data vkládaná do nového uzlu.
		 * @param node
		 *            reference na následující uzel v seznamu.
		 */
		public ListNode(Object object, ListNode node) {
			data = object;
			nextNode = node;
		}

		/**
		 * Vrátí data z uzlu.
		 * 
		 * @return reference na data v uzlu.
		 */
		public Object getObject() {
			return data;
		}

		/**
		 * Vrátí následující uzel seznamu.
		 * 
		 * @return reference na následující uzel seznamu.
		 */
		public ListNode getNext() {
			return nextNode;
		}

	}

	/**
	 * První uzel seznamu.
	 */
	private ListNode firstNode;

	/**
	 * Poslední uzel seznamu.
	 */
	private ListNode lastNode;

	/**
	 * Označení uzlu použité ve výpise.
	 */
	private String name;

	/**
	 * Vytvoří prázdný seznam s implicitním jménem "seznam".
	 */
	public List() {
		this("seznam");
	}

	/**
	 * Vytvoří prázdný seznam s daným jménem.
	 * 
	 * @param listName
	 *            jméno vytvářeného seznamu.
	 */
	public List(String listName) {
		name = listName;
		firstNode = null;
		lastNode = null;
	}

	/**
	 * Vloží daný prvek na začátek seznamu.
	 * 
	 * @param insertItem
	 *            vkládaný prvek.
	 */
	public void insertAtFront(Object insertItem) {
		if (isEmpty()) {
			firstNode = new ListNode(insertItem);
			lastNode = firstNode;
		} else {
			firstNode = new ListNode(insertItem, firstNode);
		}
	}

	/**
	 * Vloží daný prvek na konec seznamu.
	 * 
	 * @param insertItem
	 *            vkládaný prvek.
	 */
	public void insertAtBack(Object insertItem) {
		if (isEmpty()) {
			firstNode = new ListNode(insertItem);
			lastNode = firstNode;
		} else {
			lastNode.nextNode = new ListNode(insertItem);
			lastNode = lastNode.nextNode;
		}
	}

	/**
	 * Odstraní první prvek seznamu.
	 * 
	 * @return odstraněný prvek.
	 * @throws EmptyListException
	 *             pokud je seznam prázdný.
	 */
	public Object removeFromFront() throws EmptyListException {
		if (isEmpty()) {
			throw new EmptyListException(name);
		}
		// získání dat z odstraňovaného uzlu
		Object removedItem = firstNode.data;
		// aktualizace referencí firstNode a lastNode
		if (firstNode == lastNode) {
			firstNode = null;
			lastNode = null;
		} else {
			firstNode = firstNode.nextNode;
		}
		// vrácení odstraněných dat
		return removedItem;
	}

	/**
	 * Odstraní poslední prvek seznamu.
	 * 
	 * @return odstraněný prvek.
	 * @throws EmptyListException
	 *             pokud je seznam prázdný.
	 */
	public Object removeFromBack() throws EmptyListException {
		if (isEmpty()) {
			throw new EmptyListException(name);
		}
		// získání dat z odstraňovaného uzlu
		Object removedItem = lastNode.data;
		// aktualizace referencí firstNode a lastNode
		if (firstNode == lastNode) {
			firstNode = null;
			lastNode = null;
		} else {
			ListNode current = firstNode;

			while (current.nextNode != lastNode) {
				current = current.nextNode;
			}

			lastNode = current;
			current.nextNode = null;
		}
		// vrácení odstraněných dat
		return removedItem;
	}

	/**
	 * Vrací true, pokud je seznam prázdný, jinak false.
	 * 
	 * @return true, pokud je seznam prázdný, jinak false.
	 */
	public boolean isEmpty() {
		return firstNode == null;
	}

	/**
	 * Vypíše na standardní výstup obsah seznamu.
	 */
	public void print() {
		if (isEmpty()) {
			System.out.printf("%s neobsahuje žádné prvky\n", name);
			return;
		}

		System.out.printf("%s obsahuje: ", name);

		ListNode current = firstNode;

		while (current != null) {
			System.out.printf("%s ", current.data);
			current = current.nextNode;
		}

		System.out.println("\n");
	}
}
